// Copyright (c) 2023 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#pragma once
#include "scene/Scene.hpp"
#include "scene/components/display/CameraTexture.hpp"
#include "prefabs/SimpleSpinningCubePrefab.hpp"
#include "prefabs/VNTextbox.hpp"
#include "display/TextureRenderTarget.hpp"
#include "scene/components/ui/UIImage.hpp"

namespace Dawn {
  class HelloWorldScene : public Scene {
    protected:
      Camera *camera;
      Camera *camNew;
      UICanvas *canvas;
      TextureRenderTarget *renderTarget;
      Texture text;

      int32_t test = 0;

      void stage() override {
        canvas = UICanvas::create(this);
        
        camera = Camera::create(this);
        glm::vec3 off = glm::vec3(90000, 100, 100);
        camera->transform->lookAt(glm::vec3(10, 10, 10) + off, glm::vec3(0, 0, 0) + off);

        camNew = Camera::create(this);
        auto camTexture = camNew->item->addComponent<CameraTexture>();
        camNew->fov = 0.436332f;
        camNew->transform->lookAt(glm::vec3(10, 10, 10), glm::vec3(0, 0, 0));
        camTexture->renderTarget.setSize(1024, 1024);

        auto cube = SimpleSpinningCubePrefab::create(this);

        auto textbox = VNTextbox::create(this);
        textbox->transform.setParent(canvas->transform);

        auto uiTest = this->createSceneItem();
        uiTest->transform.setParent(canvas->transform);
        auto image = uiTest->addComponent<UIImage>();
        image->texture = camTexture->renderTarget.getTexture();
        image->alignment = glm::vec4(0, 0, 50, 0);
        image->alignX = UI_COMPONENT_ALIGN_START;
        image->alignUnitRight = UI_COMPONENT_ALIGN_UNIT_PERCENT;
        image->alignY = UI_COMPONENT_ALIGN_STRETCH;

        struct Color colors[] = {
          COLOR_BLUE, COLOR_MAGENTA, COLOR_WHITE,
          COLOR_MAGENTA, COLOR_CORNFLOWER_BLUE, COLOR_MAGENTA,
          COLOR_BLACK, COLOR_MAGENTA, COLOR_BLUE
        };
        text.setSize(3, 3);
        text.buffer(colors);
        textbox->border->texture = &text;
      }
      
      std::vector<Asset*> getRequiredAssets() override {
        auto assMan = &this->game->assetManager;
        std::vector<Asset*> assets;
        vectorAppend(&assets, SimpleSpinningCubePrefab::prefabAssets(assMan));
        vectorAppend(&assets, VNTextbox::prefabAssets(assMan));
        return assets;
      }

    public:
      HelloWorldScene(DawnGame *game) : Scene(game) {}
  };
}